use crate::extensions::FidoExtension;
use crate::FidoResult;
use crate::{FidoAssertionRequest, FidoCredentialRequest};
use cbor_codec::value::Value;
use num_traits::ToPrimitive;

#[derive(Copy, Clone, Debug, FromPrimitive, ToPrimitive, PartialEq)]
pub enum CredProtectLevel {
    UvOptional = 0x01,
    UvOptionalWithCredentialIDList = 0x02,
    UvRequired = 0x03,
}

impl CredProtectLevel {
    fn extension_input(self) -> Value {
        Value::U8(self.to_u8().unwrap())
    }
}

pub struct CredProtect(Value);

impl CredProtect {
    pub fn level(level: CredProtectLevel) -> Self {
        Self(level.extension_input())
    }

    fn extension_name() -> &'static str {
        "credProtect"
    }

    fn extension_input(&self) -> &Value {
        &self.0
    }
}

impl FidoExtension for CredProtect {
    fn extension_name() -> &'static str {
        CredProtect::extension_name()
    }

    fn patch_assertion_request<'a, 'b>(
        &'b self,
        _request: &mut FidoAssertionRequest<'a, 'b>,
    ) -> FidoResult<()> {
        Ok(())
    }

    fn patch_credential_request<'a>(
        &'a self,
        request: &mut FidoCredentialRequest<'a>,
    ) -> FidoResult<()> {
        request
            .extension_data
            .insert(CredProtect::extension_name(), self.extension_input());
        Ok(())
    }
}
