pub mod hmac;
pub use hmac::*;
mod cred_protect;
use crate::cbor::AuthenticatorData;
use crate::{FidoAssertionRequest, FidoCredentialRequest, FidoResult};
pub use cred_protect::*;

pub trait FidoExtension {
    fn extension_name() -> &'static str;
    fn patch_assertion_request<'a, 'b>(
        &'b self,
        request: &mut FidoAssertionRequest<'a, 'b>,
    ) -> FidoResult<()>;
    fn patch_credential_request<'a>(
        &'a self,
        request: &mut FidoCredentialRequest<'a>,
    ) -> FidoResult<()>;
}

pub trait FidoExtensionResponseParser {
    type Output;
    fn parse_response(&self, response: &AuthenticatorData) -> FidoResult<Self::Output>;
}

pub trait FidoExtensionResponseParserExt<Ext: FidoExtensionResponseParser> {
    fn parse_extension_data(&self, extension: &Ext) -> FidoResult<Ext::Output>;
}

impl<Ext: FidoExtensionResponseParser> FidoExtensionResponseParserExt<Ext> for AuthenticatorData {
    fn parse_extension_data(
        &self,
        extension: &Ext,
    ) -> FidoResult<<Ext as FidoExtensionResponseParser>::Output> {
        extension.parse_response(self)
    }
}
